{
 Cfg stuff

 (c) 1999-2000 Diman Petrov [2:5023/11.85, DimanInc@mail.ru]

 history:
 ~~~~~~~
 [03/01/00] - some little changes :*)
 [14/01/00] - tTextCfg.ParseString,
              tTextCfg.ParseStringHp,
              tTextCfg.ParseBoolean added
 [16/01/00] - tTextCfg.ParseNumber added
 [17/01/00] - Include-support added,
              constructor tTextCfg.Init added,
              constructor tTextCfg.Parse became a procedure,
              tTextCfg.Load added,
              some more fields and methods added
 [18/01/00] - tTextCfg.PrintOut added,
              tTextCfg.Error renamed to tTextCfg.ErrorOut
 [18/04/00] - tTextCfg was little reKOLBASed ;)
}
{&LocInfo+}
{$B-,D+,F-,I-,L+,R-,V-}
Unit Dp_Cfg;

interface

uses
  Dp_Basis, Dp_Comn,
  Dos, Objects;

const
  DefaultIncludeWord = 'INCLUDE';

type

{ * tTextCfg * }
{ͻ}
  pTextCfg = ^tTextCfg;
  tTextCfg = object(tObject)
    CfgName       : String;   { Config filename     }
    Line          : Longint;  { Current line number }
    ErrorsCount   : Longint;  { Errors count        }
    IncludesCount : Longint;  { Include files count }
    AllowIncludes : Boolean;  { Allow includes      } { Default: True      }
    IncludeWord   : String;   { Include Keyword     } { Default: 'Include' }
    ParsedCfgs    : pStCollection;

    constructor Init;
    destructor  Done; virtual;

    procedure   EnableIncludes(const AAllowIncludes: Boolean);
    procedure   SetIncludeKeyword(const AIncludeKeyword: String);
    procedure   Load(const ACfgName: String;
                    { Config filename      }
                     ACommentChars: tCharSet;
                    { Set of comment chars }
                     AUseDoubleSlashAsComment: Boolean);
                    { Use '//' as comment  }

    procedure   Parse; virtual;

    procedure   PrintOut  (const Message: String); virtual;
    procedure   WarningOut(const Message: String); virtual;
    procedure   ErrorOut  (const Message: String); virtual;

    procedure   ParseLine(S: String); virtual;

    procedure   ParseString  (const Value: String; var S: String);
    procedure   ParseStringHp(const Value: String; var S: String);
    procedure   ParseBoolean (const Keyword: String; Value: String; var B: Boolean);
    procedure   ParseNumber  (const Keyword, Value: String; var N: Longint);
   public
    function    CheckValue(Keyword, Value: String): Boolean; virtual;
    procedure   BadValue(const Keyword, Value: String); virtual;
   private
    Cfg           : Text;
    DoubleSl      : Boolean;
    CommentCh     : tCharSet;
    Inc1, Inc2    : pStCollection;

    procedure   NewInclude(const IncludeCfgName: String);
  end;
{ͼ}

{}
{}
{}

implementation

{ * tTextCfg * }
{ͻ}
constructor tTextCfg.Init;
begin
  inherited Init;

  AllowIncludes := True;
  IncludeWord   := DefaultIncludeWord;
  IncludesCount := 0;

  Inc1 := New(pStCollection, Init(1, 1));
  Inc2 := nil;

  ParsedCfgs := New(pStCollection, Init(1, 1));
end;
{}
destructor tTextCfg.Done;
begin
  if Inc1 <> nil then Dispose(Inc1, Done);
  if Inc2 <> nil then Dispose(Inc2, Done);
  if ParsedCfgs <> nil then Dispose(ParsedCfgs, Done);

  inherited Done;
end;
{}
procedure tTextCfg.EnableIncludes(const AAllowIncludes: Boolean);
begin
  AllowIncludes := AAllowIncludes;
end;
{}
procedure tTextCfg.SetIncludeKeyword(const AIncludeKeyword: String);
begin
  IncludeWord := StrUpCase(AIncludeKeyword);
end;
{}
procedure tTextCfg.Load(const ACfgName: String;
                        ACommentChars: tCharSet;
                        AUseDoubleSlashAsComment: Boolean);
  procedure ResetValues;
  begin
    Line          := -1;
    IncludesCount := 0;
  end;
  procedure DoIt;
  begin
    if not ftOpen(Cfg, CfgName) then ErrorOut('Unable to open '+CfgName);
    Line := 0;

    Parse;

    Close(Cfg);
  end;
var
  i: xInt;
  Was: Boolean;
begin
  CommentCh   := ACommentChars;
  DoubleSl    := AUseDoubleSlashAsComment;
  ErrorsCount := 0;
  Was         := False;

  while (IncludesCount > 0)or(not Was) do
  begin
    if Was then
    begin
      Inc2 := Inc1;
      Inc1 := New(pStCollection, Init(1, 1));

      for i := 0 to pred(IncludesCount) do
      begin
        ResetValues;

        CfgName := FExpand(pString(Inc2^.At(i))^);

        if ParsedCfgs^.IndexOfNoCase(@CfgName) <> -1 then Continue;
        ParsedCfgs^.Insert(NewStr(CfgName));

        DoIt;
      end;

      Dispose(Inc2, Done);
      Inc2 := nil;
    end
    else
    begin
      ResetValues;

      CfgName := StrUpCase(FExpand(ACfgName));

      ParsedCfgs^.Insert(NewStr(CfgName));

      DoIt;
    end;

    Was := True;
  end;
end;
{}
procedure tTextCfg.Parse;
var
  S: String;
begin
  while not EOF(Cfg) do
  begin
    Inc(Line);

    ResetIO;

    Readln(Cfg, S);
    if not CheckIO then ErrorOut('Unable to read '+CfgName);

    TrimEx(S);
    if ( S[1] in CommentCh )or( Length(S) = 0 )or
       ( (DoubleSl)and(Copy(S, 1, 2) = '//') ) then Continue;

    ParseLine(S);
  end;
end;
{}
procedure tTextCfg.PrintOut(const Message: String);
begin
  if Line = -1 then PrintlnStr(Message)
  else PrintlnStr('['+CfgName+', line '+N2Str(Line)+']: '+Message);
end;
{}
procedure tTextCfg.WarningOut(const Message: String);
begin
  if Line = -1 then Warning(Message)
  else Warning('['+CfgName+', line '+N2Str(Line)+']: '+Message);
end;
{}
procedure tTextCfg.ErrorOut(const Message: String);
begin
  if Line = -1 then Fatal(Message)
  else Fatal('['+CfgName+', line '+N2Str(Line)+']: '+Message);
end;
{}
procedure tTextCfg.ParseLine(S: String);
var
  Keyword, Value : String;
  i              : Byte;
begin
  Keyword := StrUpCase(WordExtract(S, SpaceCh, 1));
  Value   := GetAllAfterSpace(S, 1);

  for i := 1 to Length(Value) do
    if ( Value[i] in CommentCh )or
       ( (DoubleSl)and(S[i] = '/')and(S[succ(i)] = '/')and(i < Length(S)) )
    then begin
           Value := System.Copy(Value, 1, pred(i));
           Break;
         end;

  TrimEx(Value);

  if (AllowIncludes)and(Keyword = IncludeWord) then NewInclude(Value)
  else if not CheckValue(Keyword, Value) then
       begin
         Inc(ErrorsCount);
         BadValue(Keyword, Value);
       end;
end;
{}
procedure tTextCfg.ParseString(const Value: String; var S: String);
begin
  S := UnQuoteStr(Value);
end;
{}
procedure tTextCfg.ParseStringHp(const Value: String; var S: String);
var
  i: Integer;
begin
  S := UnQuoteStr(Value);

  for i := 1 to Length(S) do
    if S[i] = #$8D then S[i] := #$48
    else if S[i] = #$E0 then S[i] := #$70;
end;
{}
procedure tTextCfg.ParseBoolean(const Keyword: String; Value: String; var B: Boolean);
begin
  StrUpCaseEx(Value);
  repeat
    if (Value = 'YES')or(Value = 'ON') then begin B := True;  Break end;
    if (Value = 'NO')or(Value = 'OFF') then begin B := False; Break end;

    Inc(ErrorsCount);
    WarningOut('Wrong value of "'+Keyword+'"');
  until True;
end;
{}
procedure tTextCfg.ParseNumber(const Keyword, Value: String; var N: Longint);
var
  Code: xInt;
  Num: Longint;
begin
  Val(Value, Num, Code);

  if Code <> 0 then
  begin
    Inc(ErrorsCount);
    WarningOut('Wrong value of "'+Keyword+'"');
  end;

  N := Num;
end;

{ * Methods, which must be overriden * }
{}
function tTextCfg.CheckValue(Keyword, Value: String): Boolean;
begin
  Abstract;
end;
{}
procedure tTextCfg.BadValue(const Keyword, Value: String);
begin
  Abstract;
end;

{ * Private methods * }
{}
procedure tTextCfg.NewInclude(const IncludeCfgName: String);
var
  S: String;
begin
  S := StrUpCase(IncludeCfgName);
  if Inc1^.IndexOf(@S) = -1 then
  begin
    Inc(IncludesCount);
    Inc1^.Insert(NewStr(S));
  end;
end;
{ͼ}

end.
