{
 Log stuff

 (c) 1999-2000 Diman Petrov [2:5023/11.85, DimanInc@mail.ru]

 history:
 ~~~~~~~
 [27/02/00] - tLogBasis.Cut little rewritten
 [06/03/00] - tLogBasis.Cut fixed
}
{&LocInfo+}
{$B-,D+,F-,I-,L+,R-,V-}
Unit Dp_Log;

interface

uses
  Dp_Basis, Dp_Comn,
  Objects;

type

{ * tLogBasis * }
{ͻ}
  pLogBasis = ^tLogBasis;
  tLogBasis = object(tObject)
    LogName : String;
    Log     : Text;
    AlwaysFlush,
    Loggin  : Boolean;
    constructor Init(const ALogName: String; ALoggin, AAlwaysFlush: Boolean);
                { use AlwaysFlush, if you want to flush buffers after }
                { every writing to log                                }
    constructor Cut(const ALogName: String; const ANDays: Longint);
                { cuts log and remains info only of last NDAYS days,  }
                { including today                                     }
                { i.e.: NDays = 1 -> remain only today info           }
                {       NDays = 2 -> remain today and yesterday info  }
    destructor  Done; virtual;
    procedure   Write(const Ch: Char; const Id, S: String); virtual;
    procedure   PureWrite(const S: String); virtual;
    procedure   OnlyFileWrite(const Ch: Char; const Id, S: String); virtual;
    procedure   WarningOut(const Id, Message: String); virtual;
    procedure   Error(const Id, Message: String); virtual;
    procedure   AbortOut(const Id, Message: String; CodeExit: Byte); virtual;
   private
    NDays : Longint;
  end;
{ͼ}

{}
{}
{}

implementation

{ * tLogBasis * }
{ͻ}
constructor tLogBasis.Init(const ALogName: String;
                           ALoggin, AAlwaysFlush: Boolean);
begin
  inherited Init;

  LogName := ALogName;
  Loggin  := ALoggin;
  AlwaysFlush := AAlwaysFlush;

  if (Length(LogName) = 0)and(Loggin) then Warning('Log filename not defined');

  if Loggin then
  begin
    if not ftAppend(Log, LogName) then
      if not ftCreate(Log, LogName) then
        Warning('Unable to create file "'+LogName+'"');
    Writeln(Log);
  end;
end;
{}
constructor tLogBasis.Cut(const ALogName: String; const ANDays: Longint);
var
  Line    : Longint;
  TmpName : String;
  Out     : Text;
  Y, M, D : xWord;
  CStamp,
  Stamp   : Longint;
  S, St   : String;
  Mon     : String[3];
  DoSmth  : Boolean;
begin
  inherited Init;

  if Length(ALogName) = 0 then
  begin
    Warning('Log filename not defined');
    Exit;
  end;

  LogName := ALogName;
  NDays   := Abs(ANDays);
  Loggin  := False;

  Line    := 0;
  TmpName := GetTempFileName;
  DoSmth  := False;

  if not ftOpen(Log, LogName) then
  begin
    Warning(LogName+' not found');
    Exit;
  end;

  _GetDate(D, M, Y);
  CStamp := DaysPast(D, M, Y);
  Dec(CStamp, NDays);

  if CStamp <= 0 then
  begin
    Rewrite(Log);
    Close(Log);
    Exit;
  end;

  while not EOF(Log) do
  begin
    Readln(Log, St);                 {Date & time format: 01 Jan 70 00:00:00}
    inc(Line);
    if Length(St) = 0 then Continue;

    S := Copy(St, 3, 18);

    D := Str2xWord(WordExtract(S, SpaceCh, 1));

    M := Str2Month(WordExtract(S, SpaceCh, 2));

    if M = $FF then
    begin
      Warning('File '+LogName+' (line '+N2Str(Line)+'): wrong month');
      Exit;
    end;

    Y := Str2xWord(WordExtract(S, SpaceCh, 3));
    Y := DoubleToY2(Y);

    Stamp := DaysPast(D, M, Y);
    if CStamp < Stamp then
    begin
      DoSmth := True;
      Break;
    end;
  end;

  if DoSmth then
  begin
    ftCreate(Out, TmpName);
    Writeln(Out, St);

    while not EOF(Log) do
    begin
      Readln (Log, St);
      Writeln(Out, St);
    end;

    Close(Out);
  end;

  Close(Log);

  ResetIO;

  Erase(Log);
  if DoSmth then fMove(TmpName, LogName)
  else fFlag(LogName);

  ResetIO;
end;
{}
destructor tLogBasis.Done;
begin
  if Loggin then Close(Log);

  inherited Done;
end;
{}
procedure tLogBasis.Write(const Ch: Char; const Id, S: String);
begin
  if Loggin then
  begin
    ResetIO;
    Writeln(Log, Ch+' '+__DateTime__+' '+Id+' '+S);

    if not CheckIO then Warning('Unable to write to file '+LogName);

    if AlwaysFlush then Flush(Log);
  end;
  PrintlnStr(S);
end;
{}
procedure tLogBasis.PureWrite(const S: String);
begin
  if Loggin then
  begin
    ResetIO;
    Writeln(Log, S);

    if not CheckIO then Warning('Unable to write to file '+LogName);

    if AlwaysFlush then Flush(Log);
  end;
  PrintlnStr(S);
end;
{}
procedure tLogBasis.OnlyFileWrite(const Ch: Char; const Id, S: String);
begin
  if Loggin then
  begin
    ResetIO;
    Writeln(Log, Ch+' '+__DateTime__+' '+Id+' '+S);

    if not CheckIO then Warning('Unable to write to file '+LogName);

    if AlwaysFlush then Flush(Log);
  end;
end;
{}
procedure tLogBasis.WarningOut(const Id, Message: String);
begin
  if Loggin then
  begin
    ResetIO;
    Writeln(Log, '? '+__DateTime__+' '+Id+' Warning: '+Message);

    if not CheckIO then Warning('Unable to write to file '+LogName);

    if AlwaysFlush then Flush(Log);
  end;
  Warning(Message);
end;
{}
procedure tLogBasis.Error(const Id, Message: String);
begin
  if Loggin then
  begin
    ResetIO;
    Writeln(Log, '! '+__DateTime__+' '+Id+' Error: '+Message);

    if not CheckIO then Warning('Unable to write to file '+LogName);

    Close(Log);
  end;

  Fatal(Message);
end;
{}
procedure tLogBasis.AbortOut(const Id, Message: String; CodeExit: Byte);
begin
  if Loggin then
  begin
    ResetIO;
    Writeln(Log, '! '+__DateTime__+' '+Id+Message);

    if not CheckIO then Warning('Unable to write to file '+LogName);

    Close(Log);
  end;

  Abort(Message, CodeExit);
end;
{ͼ}

end.
