{$B-,D+,I-,L+}
{&LocInfo+}
uses
  Objects, DOS, Memory,
  skCommon, skComnTV, skMHL, skOpen,
  Dp_Basis, Dp_Comn, Dp_Error,
  mhlStuff,
  Leaker;

const
  Pid       = 'New tic analyser';
  Version   = 'v2.1';
  Author    = '(q) Diman Petrov [2:5023/11.85]';
  LongPid   = Pid + ' ' + OSid + ' ' + Version;
  FullPid   = Pid + ' ' + OSid + ' ' + Version + ' ' + Author;

  sOOM      = 'Out of memory!';

type
{[tNFObj]ͻ}
  pNFObj = ^tNFObj;
  tNFObj = object(tObject)
    Area   : pString;
    FName  : pString;
    Desc   : pString;
    Size   : Longint;
    Origin : pAddress;
    Uplink : pAddress;

    constructor Init(
                     const AArea, AFName, ADesc: String;
                     const ASize: Longint;
                     const AOrigin, AUpLink: tAddress
                    );
    destructor  Done; virtual;
  end;
{ͼ}

{[tNFCollection]ͻ}
  pNFCollection = ^tNFCollection;
  tNFCollection = object(tSortedCollection)
    function Compare(Key1, Key2: Pointer): xInt; virtual;
  end;
{ͼ}

{}
{}
{}

{ͻ}
constructor tNFObj.Init(
                        const AArea, AFName, ADesc: String;
                        const ASize: Longint;
                        const AOrigin, AUpLink: tAddress
                       );
begin
  Area  := NewStr(StrUpCase(AArea));
  FName := NewStr(LongFName2DosFName(AFName));
  Desc  := NewStr(ADesc);

  Size  := ASize;

  New(Origin);
  Origin^ := AOrigin;

  New(UpLink);
  UpLink^ := AUpLink;
end;
{}
destructor tNFObj.Done;
begin
  DisposeStr(Area);
  DisposeStr(FName);
  DisposeStr(Desc);

  Dispose(Origin);
  Dispose(UpLink);
end;
{ͼ}

{[tNFCollection]ͻ}
function tNFCollection.Compare(Key1, Key2: Pointer): xInt;
var
  P1, P2: pString;
  i     : xInt;
begin
  P1 := pNFObj(Key1)^.FName;
  P2 := pNFObj(Key2)^.FName;

  Compare := 0;
  i := 0;

  while (i < Length(GetPString(P1)))and(i < Length(GetPString(P1))) do
  begin
    inc(i);

    if P1^[i] = P2^[i] then Continue;

    if P1^[i] > P2^[i] then Compare := 1
    else Compare := -1;

    Break;
  end;
end;
{ͼ}

{[HELP]ͻ}
procedure Help;
begin
  PrintlnStr(NL+LongPid);
  PrintlnStr('Syntax: NewTic.exe <Inbound-Dir> <ID><BasePath> <FTN-Addr>');
  PrintlnStr('    <Inbound-Dir> - Dir, where *.tic files would be looked for');
  PrintlnStr('    <ID>          - Type of Base [S - Squish, J - JAM, M - Msg]');
  PrintlnStr('    <BasePath>    - Path to base to post results in it');
  PrintlnStr('    <FTN-Addr>    - Your FTN-address');
  Halt(1);
end;
{ͼ}

var
  F     : Text;
  Dir   : String;
  Base  : pMessageBase;
  BN    : String;
  S, St : String;
  Addr  : tAddress;
  C     : pSortedCollection;

{[PREPARE]ͻ}
procedure Prepare;
begin
  if ParamCount <> 3 then Help;

  PrintlnStr(NL+LongPid+' started:');
  PrintlnStr('  Preparing...');

  Dir := ParamStr(1);
  BN  := ParamStr(2);

  if not StrToAddress(ParamStr(3), Addr) then Fatal('Wrong FTN-Addr');
  if not DirExists(Dir) then Fatal(Dir+' doesn''t exist');

  InitMemory;

  InstallMHLCommon;

  if not OpenOrCreateMessageBase(Base, BN) then
    Fatal('Unable to open/create messagebase '+Copy(BN, 2, 255));

  Base^.SetFlag(afNetmail, True);

  C := New(pNFCollection, Init($10, $10));

  if LowMemory then Fatal(sOOM);
end;
{ͼ}

{[PROCESS TIC]ͻ}
procedure ProcessTic(const FN: String);
var
  Area,
  FName,
  Desc   : String;
  Size   : Longint;
  Origin,
  Uplink : tAddress;
begin
  if not ftOpen(F, FN) then
  begin
    Warning('Unable to open '+FN);
    Exit;
  end;

  PrintlnStr('   >Processing "'+StrUpCase(FN)+'"...');

  Area  := '';
  FName := '';
  Desc  := '';
  Size  := 0;

  while not EOF(F) do
  begin
    Readln(F, S);
    TrimEx(S);

    St := StrUpCase(WordExtract(S, [' '], 1));
    S  := GetAllAfterSpace(S, 1);

    if St = 'AREA' then
    begin
      Area := S;
      Continue;
    end;

    if St = 'FILE' then
    begin
      FName := S;
      Continue;
    end;

    if St = 'SIZE' then
    begin
      Size := Str2Long(S);
      Continue;
    end;

    if St = 'ORIGIN' then
    begin
      StrToAddress(S, Origin);
      Continue;
    end;

    if St = 'FROM' then
    begin
      StrToAddress(S, Uplink);
      Continue;
    end;

    if St = 'DESC' then
    begin
      Desc := S;
      Continue;
    end;
  end;

  if Area  = '' then Area  := 'UNKNOWN';
  if FName = '' then FName := 'UNTITLED';
  if Desc  = '' then Desc  := '-= no description =-';

  C^.Insert(New(pNFObj, Init(Area, FName, Desc, Size, Origin, Uplink)));

  if LowMemory then Fatal(sOOM);

  Close(F);
end;
{ͼ}

{[PROCESS TICS]ͻ}
procedure ProcessTics;
var
  SR: SearchRec;
  FN: String;
begin
  AddBackSlash(Dir);

  if fMaskCount(Dir+'*.ti?') = 0 then
    Abort('  No tics in '+Dir+NL+'  Exiting...', 255);

  Dos.FindFirst(Dir+'*.ti?', AnyFile, SR);

  while DosError = 0 do
  begin
    FN := SR.Name;

    if (StrUpCase(Path2Ext(FN)) = '.TIF')or(SR.Attr and Directory <> 0) then
    begin
      Dos.FindNext(SR);
      Continue;
    end;

    ProcessTic(Dir+FN);

    Dos.FindNext(SR);
  end;

  FindClose(SR);
end;
{ͼ}

{[PRINT TO BASE]ͻ}
procedure Print2Base(S: String); far;
begin
  Base^.PutString('  '+S);
end;
{ͼ}

{[PROCESS MSG]ͻ}
procedure ProcessMsg;
const
  Total: Longint = 0;
  Count: Longint = 0;
  {}
  procedure Print;
    {}
    procedure PrintEvery(p: pNFObj); far;
    begin
      with p^ do
      begin
        Inc(Total, Size);
        Inc(Count);

        S := '>' + RPad(GetPString(FName), 13)
                 + LPad(N2FmtStr(Size), 10)
                 + LPad('[Area: '+GetPString(Area)+']', 28)
                 + LPad('[From: '+AddressToStrEx(Origin^)+']', 27);

        Base^.PutString(S);

        WordWrap(Desc^, SpaceCh, MaxStrLen - 2, Print2Base);
      end;
    end;
    {}
  begin
    with Base^ do
    begin
      CreateNewMessage;
      mhlGenerateMsg(Base^,
                     Pid, 'Sysop', 'New files',
                     @Addr, @Addr,
                     LongPid
                    );

      SetAttribute(maLocal, True);

      PutString(' New files '+NL);
      PutString('><');
    end;

    C^.ForEach(@PrintEvery);

    with Base^ do
    begin
      PutString('><');
      PutString('> Total: '+N2FmtStr(Total)+' bytes in '+N2Str(Count)+' files'+NL);
      mhlPutTearline(Base^, FullPid);

      WriteMessage;
    end;
  end;

begin
  PrintlnStr('  Posting results...');

  Print;
end;
{ͼ}

{SHUT DOWN]ͻ}
procedure ShutDown;
begin
  PrintlnStr('  Shutdowning...');

  CloseMessageBase(Base);
  Dispose(C, Done);

  DoneMemory;

  PrintlnStr(LongPid+' finished');

  PrintlnStr('[memory leak = '+LeakStr+']');
end;
{ͼ}

begin
  Prepare;
  ProcessTics;
  ProcessMsg;
  ShutDown;
end.
